/*
	ZSlider class	V0.1alpha
	
	This class will be part of the upcoming Zonic system.

	This class is copyright by Tinic Urou in 1996.
	
	If you want to use it write to: 5uro@informatik.uni-hamburg.de
*/

#include "bemain.h"

bool	notick=FALSE; 	// used to disable any message sending
						// to our window.			
						// In This example I prevent sending messies
						// for displaying the songposition
						// Otherwise the message buffer could be
						// overfilled.

class ZSliderWindow : public BWindow 
{
	public:	ZSliderWindow(BRect frame, ZSlider *slider);
};

class ZSliderView : public BView 
{
	public:	ZSliderView(BRect frame, ZSlider *slider);
	virtual void Draw(BRect update);
	private:
			ZSlider	*Slider;
};

ZSlider::
ZSlider
(
	BRect frame,
	const char *name,
	const char *label,
	const char *maxstr,
	void  (*SliderHook)(char *dest, long allowedsize, long value),
	BMessage *message,
	ulong resizeMask, 
	ulong flags
):
BControl(frame,name,label,message,resizeMask,flags)
{
	Min=0;
	Max=100;
	Act=66;
	StrSiz=255;
	MaxStr=maxstr;
	sliderhook=SliderHook;
	clickmsg=message;
}

void ZSlider::SetValue(long num)
{
	Act=num;
	if(Window()->Lock())
	{
		UpdateValue();
		Window()->Unlock();
	}
}

void	ZSlider::SetMinValue(long val){Min=val;}
void	ZSlider::SetMaxValue(long val){Max=val;}
long	ZSlider::GetValue(){return(Act);}
long	ZSlider::GetMinValue(){return(Min);}
long	ZSlider::GetMaxValue(){return(Max);}
BRect	ZSlider::GetButton(){return(temp);}
float	ZSlider::GetMaxStrLngth(){return(MaxStrLngth);}

void ZSlider::DrawButton(BView *view, BRect frame)
{
	BRect  work;
	float  inset;

	inset=frame.LeftTop().x;

	if(view->Window()->Lock())
	{
		sliderhook(Str,255,Act);
	
		view->SetHighColor(222,222,222);
		view->FillRect(frame);
	
		view->SetHighColor(255,255,255);
		view->StrokeLine(BPoint(1+inset,1),BPoint(1+inset,frame.Height()-2));
		view->StrokeLine(BPoint(1+inset,1),BPoint(frame.Width()-2+inset,1));

		view->SetHighColor(128,128,128);	
		view->StrokeLine(BPoint(2+inset,frame.Height()-1),BPoint(frame.Width()-1+inset,frame.Height()-1));
		view->StrokeLine(BPoint(frame.Width()-1+inset,2),BPoint(frame.Width()-1+inset,frame.Height()-1));

		view->SetHighColor(0,0,0,0);
		view->StrokeRect(frame);
		view->MovePenTo(BPoint(((frame.Width()-view->StringWidth(Str))/2)+inset,(frame.Height()+fontheight)/2));
		view->DrawString(Str);	

		view->Sync();

		view->Window()->Unlock();
	}
}

void ZSlider::UpdateValue()
{
	SetHighColor(222,222,222);
	FillRect(foo);
	SetHighColor(0,0,0);
	sliderhook(Str,255,Act);
	MovePenTo(BPoint(((temp.Width()-StringWidth(Str))/2),(temp.Height()+fontheight)/2));
	DrawString(Str);	
	Sync();
}

void ZSlider::AttachedToWindow()
{
	struct font_info fi;
	if(Window()->Lock())
	{
		SetFontName("Emily");
		SetFontSize(10);
		SetDrawingMode(B_OP_OVER);	
		MaxStrLngth=StringWidth(MaxStr)+4;	
		temp=BRect(Bounds().LeftTop(),BPoint(MaxStrLngth,Bounds().Height()));
		foo=BRect(Bounds().LeftTop(),BPoint(MaxStrLngth-4,Bounds().Height()-4));
		foo.OffsetBy(2,2);
		GetFontInfo(&fi);
		fontheight=fi.size;
		Window()->Unlock();
	}
}

void ZSlider::Draw(BRect update)
{
	struct font_info fi;
	
	SetHighColor(ViewColor());
	if(temp.RightTop().x<update.RightBottom().x)
	{
		FillRect(BRect(temp.RightTop(),update.RightBottom()));
	}
	
	DrawButton(this,temp);
	
	GetFontInfo(&fi);
	
	MovePenTo(BPoint(MaxStrLngth+4,(Bounds().Height()+fi.size)/2));
	if(Label())DrawString(Label());
}

void ZSlider::MouseDown(BPoint point)
{
	ZSliderWindow	*win;
	ZSliderView *view;
	BPoint	topedge,lowedge,oldpoint=point;
	ulong 	buttons;
	long 	oldval=Act;
	BRect	work;
	
	notick=TRUE;
	
	oldpoint.x-=Act;
	
	topedge=Frame().LeftTop()+Window()->Frame().LeftTop();
	lowedge=topedge;
	lowedge.y+=Bounds().Height();
	lowedge.x+=MaxStrLngth+(abs(Min)+abs(Max));
	work=BRect(topedge,lowedge);
	work.OffsetBy((Min>0 ? Act+Min : (Min-Act)),0);
	
	win = new ZSliderWindow(work,this);
	win->AddChild(view = new ZSliderView(win->Bounds(),this));
	win->Show();
	do 
	{
		snooze(10000);
		Act=(point.x-oldpoint.x);
		if(Act>Max)Act=Max;
		if(Act<Min)Act=Min;
		if(Act!=oldval)
		{
			if(win->Lock())
			{	
				view->Draw(view->Bounds());
				win->Unlock();
			}
			oldval=Act;
		}
		GetMouse(&point, &buttons);
	} 
	while ( buttons );
	if(Window()->Lock())
	{
		Draw(Bounds());
		Window()->Unlock();
	}
	win->PostMessage(B_QUIT_REQUESTED);
	if(clickmsg)Window()->PostMessage(new BMessage(clickmsg));
	
	notick=FALSE;
}

ZSliderWindow::ZSliderWindow(BRect frame, ZSlider *slider) : BWindow(frame, NULL, B_BORDERED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_ACCEPT_FIRST_CLICK)
{	
}

ZSliderView::ZSliderView(BRect frame, ZSlider *slider) : BView(frame, "", B_FOLLOW_NONE, B_WILL_DRAW)
{
	Slider=slider;	
}

void ZSliderView::Draw(BRect updateRect)
{
	long Min,Max,Act;
	BRect frame;
	SetDrawingMode(B_OP_OVER);	
	Act=Slider->GetValue();
	Max=Slider->GetMaxValue();
	Min=Slider->GetMinValue();
	frame=Slider->GetButton();
	SetHighColor(222,222,222);
	frame.OffsetBy(Act,0);
	FillRect(BRect(BPoint(0,0),frame.LeftBottom()));
	FillRect(BRect(frame.RightTop(),Bounds().RightBottom()));
	SetFontName("Emily");
	SetFontSize(10);	
	Slider->DrawButton(this,frame);
}